package puremvc.app.view
{
	import flash.display.Sprite;
	import com.as3facileexemple.skin.classic.DefaultButtonSkin2;
	import com.as3facileexemple.skin.classic.DefaultButtonSkin;

	import com.actionscriptfacile.skin.ISkin;
	import flash.utils.getDefinitionByName;
	import com.actionscriptfacile.ui.button.Button;
	import org.osflash.thunderbolt.Logger;
	import flash.events.MouseEvent;
	import flash.utils.Dictionary;
	import com.actionscriptfacile.ui.UIComponent;
	import org.puremvc.as3.interfaces.INotification;
	import puremvc.app.ApplicationFacade;
	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.patterns.mediator.Mediator;
	
	/**
	 * Permet de construire notre menu et d'interagir avec nos composants graphiques.
	 * 
	 * @author matth
	 */
	public class MenuViewMediator extends Mediator implements IMediator
    {
        public static const NAME:String = "MenuViewMediator";
 		private var sData:XML;
		
		// Permet de forcer l'inclusion des classes de Skin Button dans le swf
        public static var linkages:Array = [DefaultButtonSkin, DefaultButtonSkin2, MouseEvent.CLICK];
        
        
        public function MenuViewMediator(viewComponent:Object)
        {
        	super(NAME, viewComponent);
           	
           	// chargement des données du model - une fois la vue initialisée
            sendNotification(ApplicationFacade.COMMAND_LOAD_MENU, 'http://www.actionscript-facile.com/ressources/xml/site.xml?a='+ Math.random()*28 );

        }
        
        
 
 		/**
 		 * Transtypage du viewComponent en son type actuel.
 		 * 
 		 */
        protected function get stage():Sprite
        {
            return viewComponent as Sprite;
        }
 		
 		
 		/**
 		 * Notifications attendues.
 		 * 
 		 */
        override public function listNotificationInterests():Array
        {
			return [ApplicationFacade.LOADING_MENU_COMPLETE];
        }
 		
 		
 		/**
 		 * Gestion des notifications.
 		 * 
 		 */
        override public function handleNotification(note:INotification):void
        {
            switch (note.getName())
            {
                case ApplicationFacade.LOADING_MENU_COMPLETE:
                	Logger.debug ("LOADING_MENU_COMPLETE MenuViewMediator");
					this.build( note.getBody() as XML );
                    break;
            }
		}

		
		/**
		 * Construction du menu.
		 * 
		 */
		private function build(data:XML) : void
		{
			sData = data;
			
			var m_correspTable:Dictionary = new Dictionary();// création d'une table de correspondance
			m_correspTable['button'] = buildButton;// appel automatique de la méthode de construction d'un Button
			
			var component:UIComponent;
			var func:Function;
		
			XML.ignoreComments = true;
			XML.ignoreWhitespace = true;
			XML.ignoreProcessingInstructions = true;
			
			for each ( var xml:XML in this.sData.children() )
			{
				// on cherche la fonction de parsing du composant
				func = m_correspTable[ xml.name() ] as Function;
				// si on ne la trouve pas on continue
				if ( func == null )
					continue;
				
				// sinon on  l'éxécute
				component = func( xml ) as UIComponent;// création du composant
				// on écoute les changements qui interviennent sur le composant
				component.addEventListener(MouseEvent.CLICK, updateContent );
				// et on ajoute le composant à la scène
				stage.addChild ( component );
			}
			
			// envoie du 1er texte à la vue Content
			for each ( var xmlContent:XML in this.sData.children() )
			{
				// envoie d'une notification avec le nouveau texte à afficher
				sendNotification(ApplicationFacade.VIEW_CONTENT, xmlContent.content );		
				break;// sortie de la boucle	
			}
		}
		
		
		/**
		 * Modifie le contenu affiché
		 * 
		 */
		private function updateContent(e:MouseEvent): void 
		{
			var oButton:Button = e.target as Button;

			// recherche du texte associé au Button			
			XML.ignoreComments = true;
			XML.ignoreWhitespace = true;
			XML.ignoreProcessingInstructions = true;

			for each ( var xml:XML in this.sData.children() )
			{
				// recherche du Button cliqué par l'utilisateur
				if ( xml.labelField.attribute('text') == oButton.label)
				{
					// envoie d'une notification avec le nouveau texte à afficher
					sendNotification(ApplicationFacade.VIEW_CONTENT, xml.content );		
					break;// sortie de la boucle
				}
					
			}
			
		}
		
		/**
		 * fonction permettant de construire un composant button
		 * 
		 */
		private function buildButton( p_xml:XML ):Button
		{
			
			var prop:XML;
			var l_labelField:XML;
			var l_labelsList:XMLList = p_xml.child('labelField');
					
			var l_component:Button = new Button();
			var parameters:Array = new Array();
			var parameter:XML;
			var method:Function;
			
			// on regarde si l'objet labelField est défini dans le XML
			if ( l_labelsList.length() > 0 )
			{
				l_labelField = l_labelsList[0];// si oui on le récupère
			}
						
			// récupère le skin du fichier xml et l'applique au composant
			var definition:Class = getDefinitionByName(p_xml.attribute('skin') ) as Class;
			l_component.applySkin( new definition() as ISkin);
			l_component.resize( Number ( p_xml.attribute('width') ), Number ( p_xml.attribute('height') ) );
			
			for each( prop in p_xml.attributes() )
			{
				// on a déjà défini le width et le height
				if ( prop.name() == "width" || prop.name() == "height" )
					continue;
					
				// on vérifie que la propriété existe avant de lui appliquer la valeur définit dans le XML
				if ( l_component.hasOwnProperty( new String( prop.name() ) ) )
				{
					l_component[ new String( prop.name() ) ] = prop.valueOf();
				}
			}
			
			// si le labelField est défini
			if ( l_labelField != null )
			{
				// on boucle sur ses propriétés
				for each( prop in l_labelField.attributes() )
				{
					// on regarde si la propriété existe et le cas échéant on lui applique la valeur
					if ( l_component.labelField.hasOwnProperty( new String( prop.name() ) ) )
					{
						l_component.labelField[new String( prop.name() )] = prop.valueOf();
					}
				}
				
				for each( prop in l_labelField.children() )
				{
					
					// on regarde si le noeud est un appel de méthode
					if ( prop.name() == "callMethod" )
					{
						if ( l_component.labelField.hasOwnProperty( prop.attribute('name').toString() ) )
						{
							
							// on récupère les paramètres
							parameters = new Array();
							for each( parameter in prop.children() )
							{
								parameters.push( parameter.valueOf() );
							}
							
							method = l_component.labelField[prop.attribute('name').toString()] as Function;

							if ( method != null )
							{
								method.apply( this, parameters );// et on appelle la méthode
							}
							
						}
					}
				}
			}			
			return l_component;// renvoie une référence vers le composant créé
		}
 
    }

}
